import { Button, Col, Divider, Modal, Row, Select, Space, Spin } from "antd";
import { useDicomUploader } from "./DicomUploader";
import { Series, Study } from "./DicomUploader/util";
import { DicomTable } from "./DicomTable";
import { useDomain } from "@/hook/useDomain";
import { Typography } from "antd";
import { useState } from "react";

const { Text } = Typography;

interface DicomUploadProps {
  children?: JSX.Element;
}

/**
 * 新窗口打开ohif阅片
 */
const openOHIFViewer = (
  StudyInstanceUID: string,
  SeriesInstanceUID: string
) => {
  const target = `http://localhost:3000/viewer/${StudyInstanceUID}?SeriesInstanceUID=${SeriesInstanceUID}`;
  window.open(target, "_blank");
};

export const DicomUpload = (props: DicomUploadProps) => {
  const { UploadInput, fileCalculator, studys, isLoading } = useDicomUploader();
  const { dicomDomainService, userDomainService } = useDomain();
  const { dcmFileNum, totalFileNum, dcmFileSize } = fileCalculator;
  const [selectRows, setSelectedRows] = useState<Study[]>([]);
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [annotators, setAnnotators] = useState([]);
  const [selectAnnotatorId, setSelectAnnotatorId] = useState<string | number>(
    ""
  );

  const onUploadFiles = async (study: Study, series: Series) => {
    const { SeriesInstanceUID, subs } = series;
    const { StudyInstanceUID } = study;
    const instances = await dicomDomainService.existInPacs({
      SeriesInstanceUID,
      StudyInstanceUID,
    });
    // pacs已存在
    if (instances.length === subs.length) {
      openOHIFViewer(StudyInstanceUID, SeriesInstanceUID);
    } else {
      let fullfilled = 0;
      Promise.all(
        series.subs.map((file) =>
          dicomDomainService.upload2Pacs(file).then((res) => {
            if (["success", "AlreadyStored"].includes(res.Status)) {
              fullfilled++;
            }
          })
        )
      ).then((res) => {
        if (res.length === series.subs.length) {
          openOHIFViewer(StudyInstanceUID, SeriesInstanceUID);
        }
      });
    }
  };

  const onClickAssign = () => {
    userDomainService.getDmpAnnotators().then((res) => {
      console.log(res);
      const { data } = res;
      setIsModalOpen(true);
      setAnnotators(data.map((u) => ({ label: u.username, value: u.id })));
    });
    console.log(selectRows);
  };

  const dcmFileInfo = !!totalFileNum && (
    <Space>
      <Text type="secondary">扫描总文件数: {totalFileNum}</Text>
      <Divider type="vertical" />
      <Text type="secondary">包含: {dcmFileNum} 个dicom文件</Text>
      <Divider type="vertical" />
      <Text type="secondary">dcm文件总体积: {dcmFileSize} </Text>
    </Space>
  );

  const onAssignConfirm = () => {
    if (!selectAnnotatorId) return;
    console.log(selectAnnotatorId);
    setSelectAnnotatorId(0);
    setIsModalOpen(false);
  };

  return (
    <div style={{ padding: 20 }}>
      <Row style={{ paddingBottom: 20 }}>
        <Col span={24}>
          <Space>
            <UploadInput />
            <Button
              disabled={selectRows.length === 0}
              type="primary"
              onClick={onClickAssign}
            >
              分配
            </Button>
          </Space>
        </Col>
      </Row>
      {dcmFileInfo}
      <DicomTable
        studys={studys}
        loading={isLoading}
        onSelectedRows={(rows) => setSelectedRows(rows)}
        onUploadFiles={onUploadFiles}
      />
      <Modal
        width={320}
        title="选择标注"
        open={isModalOpen}
        closable
        cancelText="再想想"
        okText="确定"
        onOk={onAssignConfirm}
        onCancel={() => setIsModalOpen(false)}
      >
        <Select
          style={{ width: "100%", marginBottom: 20 }}
          defaultValue={selectAnnotatorId}
          onChange={(e) => setSelectAnnotatorId(e)}
          options={annotators}
        />
      </Modal>
    </div>
  );
};
